"use strict";
//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
Object.defineProperty(exports, "__esModule", { value: true });
exports.startAIExtractorJob = exports.processAIExtractorJob = exports.processAIExtractorResult = exports.getAIExtractorFileId = exports.getAIExtractorFileName = exports.getAIExtractorS3FileName = void 0;
var primeobjects_helper_util_1 = require("primeobjects-helper-util");
var auth_call_api_1 = require("../../context/auth-call-api");
var lodash_1 = require("lodash");
var util_1 = require("../../util");
var primeobjects_helper_extract_1 = require("primeobjects-helper-extract");
var getAIExtractorS3FileName = function (fieldNamePrefix, record) {
    var file = (0, primeobjects_helper_util_1.getWebLocation)(record["".concat(fieldNamePrefix, "FileUrl")]);
    return file && (file === null || file === void 0 ? void 0 : file.path);
};
exports.getAIExtractorS3FileName = getAIExtractorS3FileName;
var getAIExtractorFileName = function (fieldNamePrefix, record) {
    var s3FileName = (0, exports.getAIExtractorS3FileName)(fieldNamePrefix, record);
    if (s3FileName) {
        var fileName = s3FileName.split("/")[s3FileName.split("/").length - 1];
        if ((0, primeobjects_helper_util_1.isGuid)(fileName.split("_")[0])) {
            fileName = fileName.replace("".concat(fileName.split("_")[0], "_"), "");
        }
        return fileName;
    }
    return undefined;
};
exports.getAIExtractorFileName = getAIExtractorFileName;
var getAIExtractorFileId = function (fieldNamePrefix, record) {
    var s3FileName = (0, exports.getAIExtractorS3FileName)(fieldNamePrefix, record);
    if (s3FileName) {
        var fileName = s3FileName.split("/")[s3FileName.split("/").length - 1];
        if ((0, primeobjects_helper_util_1.isGuid)(fileName.split("_")[0])) {
            return fileName.split("_")[0];
        }
    }
    return undefined;
};
exports.getAIExtractorFileId = getAIExtractorFileId;
var processAIExtractorResult = function (record, fieldNamePrefix, mapping) {
    if (!mapping)
        return {};
    var result = {};
    (0, lodash_1.forOwn)(mapping, function (value, key) {
        //console.log({ key });
        var keyInfo = key.split("_");
        if (keyInfo.length == 2) {
            var type = keyInfo[1];
            var key_1 = keyInfo[0];
            switch (type) {
                case "LABEL": {
                    var fieldValue = (0, lodash_1.find)(record["".concat(fieldNamePrefix, "ResultForFields")], function (f) {
                        if (key_1.indexOf("...") == 0 &&
                            (0, lodash_1.endsWith)(f.key, key_1.replace("...", "")) //key end with
                        ) {
                            return (0, primeobjects_helper_util_1.isNonEmptyString)(f.label);
                        }
                        return f.key == key_1 && (0, primeobjects_helper_util_1.isNonEmptyString)(f.label);
                    });
                    if (fieldValue) {
                        result[value] = fieldValue.label;
                    }
                    break;
                }
                case "VALUE": {
                    var fieldValue = (0, lodash_1.find)(record["".concat(fieldNamePrefix, "ResultForFields")], function (f) {
                        if (key_1.indexOf("...") == 0 &&
                            (0, lodash_1.endsWith)(f.key, key_1.replace("...", "")) //key end with
                        ) {
                            return (0, primeobjects_helper_util_1.isNonEmptyString)(f.value);
                        }
                        return f.key == key_1 && (0, primeobjects_helper_util_1.isNonEmptyString)(f.value);
                    });
                    if (fieldValue) {
                        result[value] = fieldValue.value;
                    }
                    break;
                }
                default: {
                    //COLUMN
                    var cellPropName_1 = "cell".concat(parseInt(type.replace("C", "")));
                    //console.log({ cellPropName, key });
                    var fieldValue = (0, lodash_1.find)(record["".concat(fieldNamePrefix, "ResultForRows")], function (row) {
                        if (!(0, primeobjects_helper_util_1.isNonEmptyString)(row[cellPropName_1]))
                            return false;
                        if (row.key == key_1)
                            return true;
                        //try all fields
                        for (var i = 0; i < row.cellsCount; i++) {
                            // console.log({
                            //     cellKey: `cell${i + 1}Key`,
                            //     key
                            // });
                            if (row["cell".concat(i + 1, "Key")] == key_1)
                                return true;
                        }
                        return false;
                    });
                    if (fieldValue) {
                        result[value] = fieldValue[cellPropName_1];
                    }
                    break;
                }
            }
        }
    });
    return result;
};
exports.processAIExtractorResult = processAIExtractorResult;
var processAIExtractorJob = function (solution, record, fieldNamePrefix, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping) {
    if ((0, primeobjects_helper_util_1.libTrackIsOn)())
        console.log({ onProcessJob: jobId });
    onStartAPICall("ai");
    (0, lodash_1.delay)(function () {
        var retryCountKey = "processAIExtractorJob".concat(jobId);
        var retryCount = (0, primeobjects_helper_util_1.getCache)("retryCountKey", 0) + 1;
        (0, auth_call_api_1.callApi)(solution, (0, util_1.getApiEndpoint)(solution, "ai", "process-extract-text"), 
        //"http://localhost:3000/prod/process-extract-text",
        {
            jobId: jobId,
            includeLines: includeLines
        }, "POST")
            .then(function (response) {
            //status: "IN_PROGRESS"|"SUCCEEDED"|"FAILED"|"PARTIAL_SUCCESS"
            (0, primeobjects_helper_util_1.setCache)(retryCountKey, retryCount);
            switch (response.status) {
                case "IN_PROGRESS": {
                    if (retryCount > (maxRetryCount ? maxRetryCount : 5)) {
                        onErrorAPICall("ai", "RETRY_MAX_COUNT");
                        console.error(response.status);
                    }
                    else {
                        (0, exports.processAIExtractorJob)(solution, record, fieldNamePrefix, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping);
                    }
                    break;
                }
                case "SUCCEEDED": {
                    var result = (0, primeobjects_helper_extract_1.processResult)(response.data, mapping);
                    var output = result.output, fields = result.fields, rows = result.rows, lines = result.lines;
                    record["".concat(fieldNamePrefix, "ResultForFields")] = fields;
                    record["".concat(fieldNamePrefix, "ResultForRows")] = rows;
                    record["".concat(fieldNamePrefix, "ResultForLines")] = lines;
                    record["".concat(fieldNamePrefix, "Result")] = output;
                    // record[`${fieldNamePrefix}ResultForFields`] = map(
                    //     filter(result.data, (r) => r.type == "field"),
                    //     (r, index) => {
                    //         return { ...r, index: `field-${index}` };
                    //     }
                    // );
                    // record[`${fieldNamePrefix}ResultForRows`] = map(
                    //     filter(result.data, (r) => r.type == "row"),
                    //     (r, index) => {
                    //         return { ...r, index: `row-${index}` };
                    //     }
                    // );
                    // record[`${fieldNamePrefix}ResultForLines`] = map(
                    //     filter(result.data, (r) => r.type == "line"),
                    //     (r, index) => {
                    //         return { ...r, index: `line-${index}` };
                    //     }
                    // );
                    // //if it is task, we will need to use mapping to generate output
                    // record[`${fieldNamePrefix}Result`] =
                    //     processAIExtractorResult(
                    //         record,
                    //         fieldNamePrefix,
                    //         mapping
                    //     );
                    updateRecord(record);
                    onSuccessAPICall("ai");
                    if (onSuccessProcessAIExtractorJob) {
                        onSuccessProcessAIExtractorJob(record);
                    }
                    break;
                }
                default: {
                    onErrorAPICall("ai", response.status);
                    console.error(response.status);
                    break;
                }
            }
        })
            .catch(function (error) {
            (0, primeobjects_helper_util_1.setCache)(retryCountKey, retryCount);
            onErrorAPICall("ai", error);
            console.error(error);
        });
    }, 1000 * (delaySeconds ? delaySeconds : 3));
};
exports.processAIExtractorJob = processAIExtractorJob;
var startAIExtractorJob = function (solution, record, fieldNamePrefix, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping) {
    var jobId = record["".concat(fieldNamePrefix, "JobId")];
    if (jobId) {
        return (0, exports.processAIExtractorJob)(solution, record, fieldNamePrefix, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping);
    }
    onStartAPICall("ai");
    var fileS3Name = (0, exports.getAIExtractorS3FileName)(fieldNamePrefix, record);
    var fileId = (0, exports.getAIExtractorFileId)(fieldNamePrefix, record);
    (0, auth_call_api_1.callApi)(solution, (0, util_1.getApiEndpoint)(solution, "ai", "start-extract-text"), 
    // "http://localhost:3000/prod/start-extract-text",
    // `${solution.apis.ai}start-extract-text`,
    {
        fileName: fileS3Name,
        id: fileId
    }, "POST")
        .then(function (result) {
        record["".concat(fieldNamePrefix, "JobId")] = result.jobId;
        updateRecord(record);
        (0, exports.processAIExtractorJob)(solution, record, fieldNamePrefix, result.jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping);
    })
        .catch(function (error) {
        onErrorAPICall("ai", error);
        console.error(error);
    });
};
exports.startAIExtractorJob = startAIExtractorJob;
//# sourceMappingURL=ai-extractor-util.js.map