import React, { CSSProperties } from "react";
import { TTableSelectionType } from "../table";
import { TListColumn, TListColumnSettings, TListRecordInAction } from "./util";
export type TListQueryScope = "organization" | "global" | "global-or-organization" | "global-and-organization" | "global-or-co-organization" | "global-and-co-organization" | "membership";
export type TListQuery = {
    name: string;
    entityName: string;
    entityType?: string;
    keywords?: string;
    tags?: string[];
    nodes?: Record<string, any>[];
    pageNumber?: number;
    scope?: TListQueryScope;
    query?: Record<string, any>;
    recordForMembership?: Record<string, any>;
};
export type TListResult = {
    count: number;
    continuation?: any;
    data: Record<string, any>[];
};
export type TListTableSectionProps = {
    showScrollbarX?: boolean;
    minWidth?: number;
    wrapperClassName?: string;
    wrapperStyle?: CSSProperties;
    tableWrapperClassName?: string;
    tableWrapperStyle?: CSSProperties;
    query?: Record<string, any>;
    defaultRecordId?: string;
    refresh?: string;
    showTags?: boolean;
    apis?: Record<string, any>;
    recordToUpdate?: Record<string, any>;
    recordDeleted?: Record<string, any>;
    recordForMembership?: Record<string, any>;
    recordInAction?: TListRecordInAction;
    selectionType?: TTableSelectionType;
    manualLoadMore?: boolean;
    showInfoHeader?: boolean;
    infoHeader?: any;
    infoHeaderContent?: any;
    showInfoFooter?: boolean;
    infoFooter?: any;
    infoFooterContent?: any;
    renderEmpty?: () => any;
    toApplyConditions?: (conditions: Record<string, any>[]) => void;
    getColumns?: (onClick: (record: Record<string, any>, action: string, entity?: Record<string, any>, rowIndex?: number) => void, entity?: Record<string, any>, settings?: TListColumnSettings) => TListColumn[];
    onRowSelected?: (newSelectedIds: string[], newSelectedRecords: Record<string, any>[]) => void;
    onReadRecord?: (record: Record<string, any>, action?: string, entity?: Record<string, any>, rowIndex?: number) => void;
    onEditRecord?: (record: Record<string, any>, action?: string, entity?: Record<string, any>, rowIndex?: number) => void;
    onDeleteRecord?: (record: Record<string, any>, action?: string, entity?: Record<string, any>, rowIndex?: number) => void;
    onCloneRecord?: (record: Record<string, any>, action?: string, entity?: Record<string, any>, rowIndex?: number) => void;
    onCreateRecord?: (record: Record<string, any>, action?: string, entity?: Record<string, any>, rowIndex?: number) => void;
    onClickRecord?: (record: Record<string, any>, action?: string, entity?: Record<string, any>, rowIndex?: number) => void;
    onLoadPostProcess?: (query: Record<string, any>, result: TListResult) => Promise<TListResult>;
    onLoadSuccess?: (query: Record<string, any>, result: TListResult) => void;
    onLoadFailed?: (query: Record<string, any>, error: any) => void;
    onClickTag?: (tag: string) => void;
};
export declare const recordIsPartialChanged: (record: Record<string, any>) => any;
export declare const recordIsPartialSaving: (record: Record<string, any>) => any;
export declare const recordIsDeleting: (record: Record<string, any>) => any;
export declare const ListTableSection: (props: TListTableSectionProps) => React.JSX.Element;
export default ListTableSection;
