//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
import { getWebLocation, isGuid, setCache, getCache, isNonEmptyString, libTrackIsOn } from "primeobjects-helper-util";
import { callApi } from "../../context/auth-call-api";
import { delay, forOwn, find, endsWith } from "lodash";
import { getApiEndpoint } from "../../util";
import { processResult } from "primeobjects-helper-extract";
export var getAIExtractorS3FileName = function (fieldNamePrefix, record) {
    var file = getWebLocation(record["".concat(fieldNamePrefix, "FileUrl")]);
    return file && (file === null || file === void 0 ? void 0 : file.path);
};
export var getAIExtractorFileName = function (fieldNamePrefix, record) {
    var s3FileName = getAIExtractorS3FileName(fieldNamePrefix, record);
    if (s3FileName) {
        var fileName = s3FileName.split("/")[s3FileName.split("/").length - 1];
        if (isGuid(fileName.split("_")[0])) {
            fileName = fileName.replace("".concat(fileName.split("_")[0], "_"), "");
        }
        return fileName;
    }
    return undefined;
};
export var getAIExtractorFileId = function (fieldNamePrefix, record) {
    var s3FileName = getAIExtractorS3FileName(fieldNamePrefix, record);
    if (s3FileName) {
        var fileName = s3FileName.split("/")[s3FileName.split("/").length - 1];
        if (isGuid(fileName.split("_")[0])) {
            return fileName.split("_")[0];
        }
    }
    return undefined;
};
export var processAIExtractorResult = function (record, fieldNamePrefix, mapping) {
    if (!mapping)
        return {};
    var result = {};
    forOwn(mapping, function (value, key) {
        //console.log({ key });
        var keyInfo = key.split("_");
        if (keyInfo.length == 2) {
            var type = keyInfo[1];
            var key_1 = keyInfo[0];
            switch (type) {
                case "LABEL": {
                    var fieldValue = find(record["".concat(fieldNamePrefix, "ResultForFields")], function (f) {
                        if (key_1.indexOf("...") == 0 &&
                            endsWith(f.key, key_1.replace("...", "")) //key end with
                        ) {
                            return isNonEmptyString(f.label);
                        }
                        return f.key == key_1 && isNonEmptyString(f.label);
                    });
                    if (fieldValue) {
                        result[value] = fieldValue.label;
                    }
                    break;
                }
                case "VALUE": {
                    var fieldValue = find(record["".concat(fieldNamePrefix, "ResultForFields")], function (f) {
                        if (key_1.indexOf("...") == 0 &&
                            endsWith(f.key, key_1.replace("...", "")) //key end with
                        ) {
                            return isNonEmptyString(f.value);
                        }
                        return f.key == key_1 && isNonEmptyString(f.value);
                    });
                    if (fieldValue) {
                        result[value] = fieldValue.value;
                    }
                    break;
                }
                default: {
                    //COLUMN
                    var cellPropName_1 = "cell".concat(parseInt(type.replace("C", "")));
                    //console.log({ cellPropName, key });
                    var fieldValue = find(record["".concat(fieldNamePrefix, "ResultForRows")], function (row) {
                        if (!isNonEmptyString(row[cellPropName_1]))
                            return false;
                        if (row.key == key_1)
                            return true;
                        //try all fields
                        for (var i = 0; i < row.cellsCount; i++) {
                            // console.log({
                            //     cellKey: `cell${i + 1}Key`,
                            //     key
                            // });
                            if (row["cell".concat(i + 1, "Key")] == key_1)
                                return true;
                        }
                        return false;
                    });
                    if (fieldValue) {
                        result[value] = fieldValue[cellPropName_1];
                    }
                    break;
                }
            }
        }
    });
    return result;
};
export var processAIExtractorJob = function (solution, record, fieldNamePrefix, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping) {
    if (libTrackIsOn())
        console.log({ onProcessJob: jobId });
    onStartAPICall("ai");
    delay(function () {
        var retryCountKey = "processAIExtractorJob".concat(jobId);
        var retryCount = getCache("retryCountKey", 0) + 1;
        callApi(solution, getApiEndpoint(solution, "ai", "process-extract-text"), 
        //"http://localhost:3000/prod/process-extract-text",
        {
            jobId: jobId,
            includeLines: includeLines
        }, "POST")
            .then(function (response) {
            //status: "IN_PROGRESS"|"SUCCEEDED"|"FAILED"|"PARTIAL_SUCCESS"
            setCache(retryCountKey, retryCount);
            switch (response.status) {
                case "IN_PROGRESS": {
                    if (retryCount > (maxRetryCount ? maxRetryCount : 5)) {
                        onErrorAPICall("ai", "RETRY_MAX_COUNT");
                        console.error(response.status);
                    }
                    else {
                        processAIExtractorJob(solution, record, fieldNamePrefix, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping);
                    }
                    break;
                }
                case "SUCCEEDED": {
                    var result = processResult(response.data, mapping);
                    var output = result.output, fields = result.fields, rows = result.rows, lines = result.lines;
                    record["".concat(fieldNamePrefix, "ResultForFields")] = fields;
                    record["".concat(fieldNamePrefix, "ResultForRows")] = rows;
                    record["".concat(fieldNamePrefix, "ResultForLines")] = lines;
                    record["".concat(fieldNamePrefix, "Result")] = output;
                    // record[`${fieldNamePrefix}ResultForFields`] = map(
                    //     filter(result.data, (r) => r.type == "field"),
                    //     (r, index) => {
                    //         return { ...r, index: `field-${index}` };
                    //     }
                    // );
                    // record[`${fieldNamePrefix}ResultForRows`] = map(
                    //     filter(result.data, (r) => r.type == "row"),
                    //     (r, index) => {
                    //         return { ...r, index: `row-${index}` };
                    //     }
                    // );
                    // record[`${fieldNamePrefix}ResultForLines`] = map(
                    //     filter(result.data, (r) => r.type == "line"),
                    //     (r, index) => {
                    //         return { ...r, index: `line-${index}` };
                    //     }
                    // );
                    // //if it is task, we will need to use mapping to generate output
                    // record[`${fieldNamePrefix}Result`] =
                    //     processAIExtractorResult(
                    //         record,
                    //         fieldNamePrefix,
                    //         mapping
                    //     );
                    updateRecord(record);
                    onSuccessAPICall("ai");
                    if (onSuccessProcessAIExtractorJob) {
                        onSuccessProcessAIExtractorJob(record);
                    }
                    break;
                }
                default: {
                    onErrorAPICall("ai", response.status);
                    console.error(response.status);
                    break;
                }
            }
        })
            .catch(function (error) {
            setCache(retryCountKey, retryCount);
            onErrorAPICall("ai", error);
            console.error(error);
        });
    }, 1000 * (delaySeconds ? delaySeconds : 3));
};
export var startAIExtractorJob = function (solution, record, fieldNamePrefix, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping) {
    var jobId = record["".concat(fieldNamePrefix, "JobId")];
    if (jobId) {
        return processAIExtractorJob(solution, record, fieldNamePrefix, jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping);
    }
    onStartAPICall("ai");
    var fileS3Name = getAIExtractorS3FileName(fieldNamePrefix, record);
    var fileId = getAIExtractorFileId(fieldNamePrefix, record);
    callApi(solution, getApiEndpoint(solution, "ai", "start-extract-text"), 
    // "http://localhost:3000/prod/start-extract-text",
    // `${solution.apis.ai}start-extract-text`,
    {
        fileName: fileS3Name,
        id: fileId
    }, "POST")
        .then(function (result) {
        record["".concat(fieldNamePrefix, "JobId")] = result.jobId;
        updateRecord(record);
        processAIExtractorJob(solution, record, fieldNamePrefix, result.jobId, updateRecord, onStartAPICall, onSuccessAPICall, onErrorAPICall, onSuccessProcessAIExtractorJob, delaySeconds, maxRetryCount, includeLines, mapping);
    })
        .catch(function (error) {
        onErrorAPICall("ai", error);
        console.error(error);
    });
};
//# sourceMappingURL=ai-extractor-util.js.map