//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useEffect, useState } from "react";
import { cloneDeep, map, isArray, filter, isNil, unionBy } from "lodash";
import { doNothing, getColor, getTimestamp, isNonEmptyString, isObject, newGuid, getEntity } from "primeobjects-helper-util";
import ListHeaderSectionInternal from "./header";
import ListFiltersSectionInternal from "./filters";
import ListTableSectionInternal from "./table";
import ListEmptySectionInternal from "./empty";
import { DEFAULT_LIST_COLUMNS } from "./util";
import { setLocalStorage } from "primeobjects-ui-web-basic/build/cjs/util/local-storage";
import { getGlobalComponent, getApiEndpoint, getApiEndpointBase } from "../../util";
import { callApi } from "../../context/auth-call-api";
import { usePrimaryColorName, useSolution } from "primeobjects-helper-ui";
import Notification from "../../controls/antd/notification";
var ListFiltersSection = undefined;
var ListTableSection = undefined;
var ListHeaderSection = undefined;
var ListEmptySection = undefined;
var generateQuery = function (listQuery) {
    var name = listQuery.name, pageNumber = listQuery.pageNumber, keywords = listQuery.keywords, tags = listQuery.tags, nodes = listQuery.nodes, conditions = listQuery.conditions, scope = listQuery.scope, recordForMembership = listQuery.recordForMembership;
    var query = __assign(__assign({ name: name, orderBy: [{ type: "desc", attribute: "_ts" }], scope: scope ? scope : "organization", conditions: [{ attribute: "stateCode", op: ">=", value: 0 }], pageSize: 25, pageNumber: pageNumber ? pageNumber : 1 }, (listQuery.query ? listQuery.query : {})), { prevListQuery: JSON.stringify(listQuery) });
    // if (_track) console.log({ recordForMembership, scope });
    if (conditions) {
        query.conditions = filter(unionBy(query.conditions, conditions, function (v) {
            return JSON.stringify(v);
        }), function (f) {
            return f.remove != true;
        });
    }
    if (recordForMembership &&
        recordForMembership.id &&
        query.scope == "membership") {
        query.scope = "membership";
        query.recordIdForMembership = recordForMembership.id;
    }
    if (isNonEmptyString(keywords)) {
        query.keywords = keywords;
    }
    if (isArray(tags) && tags.length > 0) {
        query.tags = tags;
    }
    if (nodes && nodes.length > 0) {
        var treeFilters = {};
        for (var i = 0; i < nodes.length; i++) {
            var node = nodes[i];
            if (!treeFilters[node.attributeName])
                treeFilters[node.attributeName] = [];
            treeFilters[node.attributeName].push(node.id);
        }
        query.treeFilters = treeFilters;
    }
    return query;
};
export var ListSection = function (props) {
    var _a, _b;
    if (!ListFiltersSection) {
        console.log("Get global components in /sections/list/index.tsx");
        ListFiltersSection = getGlobalComponent("ListFiltersSection", ListFiltersSectionInternal);
        ListTableSection = getGlobalComponent("ListTableSection", ListTableSectionInternal);
        ListHeaderSection = getGlobalComponent("ListHeaderSection", ListHeaderSectionInternal);
        ListEmptySection = getGlobalComponent("ListEmptySection", ListEmptySectionInternal);
    }
    var currentRecord = props.currentRecord, recordToUpdate = props.recordToUpdate, hideHeader = props.hideHeader, createButtonText = props.createButtonText, hideSearch = props.hideSearch, hideSearchService = props.hideSearchService, showFilterButton = props.showFilterButton, showToggleTagsButton = props.showToggleTagsButton, hideRefreshButton = props.hideRefreshButton, hideCreateButton = props.hideCreateButton, hideExportButton = props.hideExportButton, showUploadButton = props.showUploadButton, showBatchButton = props.showBatchButton, exportProps = props.exportProps, hideCategoriesButton = props.hideCategoriesButton, hideEntityIcon = props.hideEntityIcon, minWidth = props.minWidth, showScrollbarX = props.showScrollbarX, apis = props.apis, recordForMembership = props.recordForMembership, defaultRecordId = props.defaultRecordId;
    var _c = useState(undefined), queryResult = _c[0], setQueryResult = _c[1];
    var wrapperClassName = isNonEmptyString(props.wrapperClassName)
        ? props.wrapperClassName
        : "";
    var wrapperStyle = isObject(props.wrapperStyle) ? props.wrapperStyle : {};
    var _d = useState({}), ui = _d[0], setUI = _d[1];
    var _e = useState(undefined), query = _e[0], setQuery = _e[1];
    var _f = useState(0), filterSectionHeight = _f[0], setFilterSectionHeight = _f[1];
    var keywords = ui.keywords;
    var tags = ui.tags;
    var nodes = ui.nodes;
    var conditions = ui.conditions;
    var solution = useSolution();
    var primaryColorName = usePrimaryColorName();
    var showTags = ui.showTags;
    var entity = query && getEntity(solution, query.entityName, query.entityType);
    var _g = useState(""), refresh = _g[0], setRefresh = _g[1];
    var _h = useState(undefined), notification = _h[0], setNotification = _h[1];
    var _j = useState(undefined), cancelSearch = _j[0], setCancelSearch = _j[1];
    var _k = useState(undefined), recordInAction = _k[0], setRecordInAction = _k[1];
    var _l = useState(undefined), recordDeleted = _l[0], setRecordDeleted = _l[1];
    var localStorageKey = "list-ui-".concat(entity === null || entity === void 0 ? void 0 : entity.entityName, "-").concat(entity === null || entity === void 0 ? void 0 : entity.entityType);
    var filters = __spreadArray(__spreadArray(__spreadArray(__spreadArray([], (keywords ? [{ type: "search", text: keywords }] : []), true), map(tags, function (tag) {
        return { type: "tag", text: tag };
    }), true), map(nodes, function (node) {
        return __assign(__assign({}, node), { type: "node" });
    }), true), map(conditions, function (condition) {
        return __assign(__assign({}, condition), { type: "condition" });
    }), true);
    var hideFilterSection = props.hideFilterSection == true || filters.length == 0;
    useEffect(function () { }, [entity === null || entity === void 0 ? void 0 : entity.entityName, entity === null || entity === void 0 ? void 0 : entity.entityType]);
    useEffect(function () {
        updateUI(__assign(__assign({}, ui), { nodes: cloneDeep(props.nodes) }));
    }, [props.nodes]);
    useEffect(function () {
        updateUI(__assign(__assign({}, ui), { tags: cloneDeep(props.tags) }));
    }, [props.tags]);
    useEffect(function () {
        if (!isNil(props.resetUI)) {
            setCancelSearch(Date.now());
            updateUI({});
        }
    }, [props.resetUI]);
    useEffect(function () {
        var newLisyQuery = {
            name: "first",
            keywords: keywords,
            tags: tags,
            nodes: nodes,
            conditions: conditions,
            recordForMembership: recordForMembership,
            query: props.query
        };
        if ((query === null || query === void 0 ? void 0 : query.prevListQuery) != JSON.stringify(newLisyQuery)) {
            setQuery(generateQuery(newLisyQuery));
        }
    }, [recordForMembership, tags, nodes, conditions, keywords, props.query]);
    // const onCreateRecord = (record: Record<string, any>) => {
    //     props.onCreateRecord && props.onCreateRecord(record);
    // };
    useEffect(function () {
        var _a;
        if ((_a = props.recordToDelete) === null || _a === void 0 ? void 0 : _a.id)
            onDeleteRecord(props.recordToDelete);
    }, [props.recordToDelete]);
    useEffect(function () {
        var _a;
        if ((_a = props.recordDeleted) === null || _a === void 0 ? void 0 : _a.id)
            setRecordDeleted(props.recordDeleted);
    }, [props.recordDeleted]);
    useEffect(function () {
        if (!isNil(props.refresh))
            onRefresh();
    }, [props.refresh]);
    var updateUI = function (newFilters) {
        setLocalStorage(localStorageKey, newFilters);
        setUI(newFilters);
    };
    var toApplyConditions = function (conditions) {
        console.log({ conditions: conditions });
        updateUI(__assign(__assign({}, ui), { conditions: conditions }));
    };
    var onSubmitSearch = function (keywords) {
        updateUI(__assign(__assign({}, ui), { keywords: keywords ? keywords : "" }));
    };
    var onClickTag = function (tag) {
        updateUI(__assign(__assign({}, ui), { tags: [tag] }));
    };
    var onRefresh = function () {
        setRecordInAction({ doing: false, record: {}, actionName: "refresh" });
        setRefresh(newGuid());
    };
    var onClickToggleTagsButton = function () {
        var showTags = !(ui.showTags == true);
        updateUI(__assign(__assign({}, ui), { showTags: showTags }));
    };
    var getCurrentRecord = function () {
        return currentRecord;
    };
    var getColumns = function (onClick, entity, settings) {
        var _a;
        var func = (_a = props.getColumns) !== null && _a !== void 0 ? _a : DEFAULT_LIST_COLUMNS;
        return func(onClick, entity, settings
            ? __assign(__assign({}, settings), { ui: ui, toApplyConditions: toApplyConditions, getCurrentRecord: getCurrentRecord }) : { ui: ui, toApplyConditions: toApplyConditions, getCurrentRecord: getCurrentRecord });
    };
    var onClickFilterButton = function () { };
    var onEditRecord = function (record, action, entity, rowIndex) {
        props.onEditRecord &&
            props.onEditRecord(cloneDeep(record), action, entity, rowIndex);
    };
    var onUploadRecord = function (record) {
        props.onUploadRecord && props.onUploadRecord(record);
    };
    var onProcessUploadRecord = function (record) {
        return (props.onProcessUploadRecord && props.onProcessUploadRecord(record));
    };
    var onBatchRecord = function (record) {
        props.onBatchRecord && props.onBatchRecord(record);
    };
    var onProcessBatchRecord = function (record) {
        return props.onProcessBatchRecord && props.onProcessBatchRecord(record);
    };
    var onDeleteRecord = function (record) {
        setRecordInAction({
            doing: true,
            record: record,
            actionName: "deleting",
            actionMessage: "Deleting ...",
            color: getColor(primaryColorName, 600)
        });
        var apiEndpoint = apis
            ? getApiEndpointBase(apis, "data", "delete")
            : getApiEndpoint(solution, "data", "delete", entity === null || entity === void 0 ? void 0 : entity.entityName);
        apiEndpoint &&
            callApi(solution, apiEndpoint, 
            //"http://localhost:3000/prod/delete",
            { id: record.id }, "DELETE")
                .then(function (result) {
                doNothing(result);
                setRecordInAction({
                    doing: false,
                    record: {},
                    actionName: "deleted-done",
                    color: getColor(primaryColorName, 600)
                });
                setNotification({
                    type: "success",
                    refresh: getTimestamp(),
                    message: "Record has been deleted successfully."
                });
                setRecordDeleted(record);
                props.onDeleteRecord && props.onDeleteRecord(record);
            })
                .catch(function (error) {
                console.error(error);
                setRecordInAction({
                    doing: false,
                    record: {},
                    actionName: "delete-failed"
                });
                setNotification({
                    type: "error",
                    refresh: getTimestamp(),
                    message: "Failed to delete the record! Please try again later."
                });
            });
    };
    var onLoadSuccess = function (query, result) {
        setQueryResult({ query: query, result: result });
        props.onLoadSuccess && props.onLoadSuccess(query, result);
    };
    var onCloneRecord = function (record, action, entity, rowIndex) {
        doNothing({ action: action, rowIndex: rowIndex });
        setRecordInAction({
            doing: true,
            record: record,
            actionName: "cloning",
            actionMessage: "Cloning ...",
            color: primaryColorName
        });
        var apiEndpoint = apis
            ? getApiEndpointBase(apis, "data", "clone")
            : getApiEndpoint(solution, "data", "clone", entity === null || entity === void 0 ? void 0 : entity.entityName);
        apiEndpoint &&
            callApi(solution, apiEndpoint, 
            //"http://localhost:3000/prod/clone",
            {
                entity: entity,
                data: record,
                displayFieldName: entity &&
                    isArray(entity.displayFields) &&
                    entity.displayFields.length > 0
                    ? entity.displayFields[0]
                    : ""
            }, "POST")
                .then(function (result) {
                setRecordInAction({
                    doing: false,
                    record: {},
                    actionName: "clone-done"
                });
                setNotification({
                    type: "success",
                    refresh: getTimestamp(),
                    message: "Record has been cloned successfully."
                });
                props.onCloneRecord && props.onCloneRecord(result);
            })
                .catch(function (error) {
                console.error(error);
                setRecordInAction({
                    doing: false,
                    record: {},
                    actionName: "clone-failed"
                });
                setNotification({
                    type: "error",
                    refresh: getTimestamp(),
                    message: "Failed to clone the record! Please try again later."
                });
            });
    };
    var onRemoveFilter = function (f) {
        var newData = [];
        switch (f.type) {
            case "search": {
                setCancelSearch(Date.now());
                break;
            }
            case "node": {
                var newNodes = filter(nodes, function (node) {
                    return node.id != f.id;
                });
                newData = newNodes;
                updateUI(__assign(__assign({}, ui), { nodes: newNodes }));
                break;
            }
            case "condition": {
                var newConditions = filter(conditions, function (c) {
                    return ("".concat(f.attribute, "-").concat(f.value, "-").concat(f.op) !=
                        "".concat(c.attribute, "-").concat(c.value, "-").concat(c.op));
                });
                newData = newConditions;
                updateUI(__assign(__assign({}, ui), { conditions: newConditions }));
                break;
            }
            case "tag": {
                newData = []; //TODO: allow remove a single tag
                updateUI(__assign(__assign({}, ui), { tags: [] }));
                break;
            }
        }
        props.onRemoveFilter && props.onRemoveFilter(f, newData);
    };
    var onClickRecord = function (record, action, entity, rowIndex) {
        props.onClickRecord &&
            props.onClickRecord(record, action, entity, rowIndex);
    };
    var renderEmptyTable = function () {
        return (React.createElement(ListEmptySection, { entity: entity, search: keywords, tags: tags, nodes: nodes, conditions: conditions }));
    };
    var onResizeFilter = function (width, height) {
        doNothing(width);
        setFilterSectionHeight((height !== null && height !== void 0 ? height : 0) + 24); //NOTE: 24 is padding, if we change the padding of filter section, we should also change 24
    };
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: "list w-full h-full flex flex-col overflow-x-hidden ".concat(wrapperClassName), style: wrapperStyle },
            !hideHeader && entity && (React.createElement(ListHeaderSection, { queryResult: queryResult, hide: true, apis: apis, currentRecord: currentRecord, entity: entity, showTags: showTags, onClickFilterButton: showFilterButton ? onClickFilterButton : undefined, onClickToggleTagsButton: showToggleTagsButton
                    ? onClickToggleTagsButton
                    : undefined, renderCategories: props.renderCategories, onClickCreateButton: props.onCreateRecord, hideCategoriesButton: hideCategoriesButton, hideCreateButton: hideCreateButton, hideEntityIcon: hideEntityIcon, hideExportButton: hideExportButton, onUploadRecord: onUploadRecord, onProcessUploadRecord: onProcessUploadRecord, showUploadButton: showUploadButton, onBatchRecord: onBatchRecord, onProcessBatchRecord: onProcessBatchRecord, showBatchButton: showBatchButton, exportProps: exportProps, hideSearchService: hideSearchService, hideSearch: hideSearch, onSubmitSearch: onSubmitSearch, onRefresh: hideRefreshButton ? undefined : onRefresh, cancelSearch: cancelSearch, createButtonText: createButtonText })),
            !hideFilterSection && entity && (React.createElement(ListFiltersSection, { wrapperStyle: {
                    borderTop: "1px solid #f0f0f0"
                }, filters: filters, onRemove: onRemoveFilter, onResize: onResizeFilter })),
            entity && (React.createElement(ListTableSection, { wrapperStyle: __assign({ height: "calc(100% - (".concat(65 +
                        (hideFilterSection ? 0 : filterSectionHeight), "px)"), borderTop: "1px solid #f0f0f0" }, ((_a = props.tableWrapperStyle) !== null && _a !== void 0 ? _a : {})), wrapperClassName: (_b = props.tableWrapperClassName) !== null && _b !== void 0 ? _b : "", showInfoHeader: props.hideInfoHeader != true, infoHeaderContent: props.infoHeaderContent, showInfoFooter: props.hideInfoFooter != true, infoFooterContent: props.infoFooterContent, show: true, apis: apis, recordInAction: recordInAction, recordToUpdate: recordToUpdate, recordDeleted: recordDeleted, onClickTag: onClickTag, showTags: showTags, refresh: refresh, onEditRecord: onEditRecord, onDeleteRecord: onDeleteRecord, onCloneRecord: onCloneRecord, toApplyConditions: toApplyConditions, onLoadPostProcess: props.onLoadPostProcess, onLoadSuccess: onLoadSuccess, onLoadFailed: props.onLoadFailed, manualLoadMore: props.manualLoadMore, defaultRecordId: defaultRecordId, query: query, getColumns: getColumns, minWidth: minWidth, showScrollbarX: showScrollbarX, onClickRecord: onClickRecord, renderEmptyTable: renderEmptyTable })),
            isNil(query) && (React.createElement("div", { className: "w-full p-4 flex flex-col text-red-600" }, "The query is not defined.")),
            !isNil(query) && !entity && (React.createElement("div", { className: "w-full p-4 flex flex-col text-red-600" }, "The entity does not exist."))),
        React.createElement(Notification, __assign({ id: "list-notification", placement: "bottom" }, notification))));
};
export default ListSection;
//# sourceMappingURL=index.js.map